#include <stdio.h>
#include <stdlib.h>

#include "aspi.h"
#include "cdrom.h"
#include "scsidefs.h"

static unsigned char cdb[12];
static int target;
static long blocksize;
static int cd_is_open = 0;
#define CDO_init 0
#define CDO_open 1
#define CDO_close 2

char *lb2hms(int blocks)
{
  static char buf[4][15];
  static int i;
  char *bp;
  int h, m, s, f;

  i = (i+1)%4;
  bp = buf[i];

  f = blocks % 75;
  blocks /= 75;
  s = blocks % 60;
  blocks /= 60;
  m = blocks % 60;
  blocks /= 60;
  h = blocks;
  if (h)
    sprintf(bp, "%d:%02d:%02d.%02d", h, m, s, f);
  else
    sprintf(bp, "%d:%02d.%02d", m, s, f);
  return bp;
}

char *
comma(int b)
{
  static char buf[20];
  static char map[] = "abc,def,ghi";
  int i;
  char *rv;
  sprintf(buf, "%9d", b);
  for (i=10; i>=0; i--)
    if (map[i] == ',')
      buf[i] = ',';
    else
      buf[i] = buf[map[i]-'a'];
  rv = buf;
  while (*rv == ' ' || *rv == ',')
    rv++;
  return rv;
}

unsigned long
swapl(unsigned long q)
{
  return ((q>>24) & 0x000000ffL)
       | ((q>> 8) & 0x0000ff00L)
       | ((q<< 8) & 0x00ff0000L)
       | ((q<<24) & 0xff000000L);
}

static void
cd_close_auto(void)
{
  cd_close(0);
}

void
check_sense(int ignore, int s, char *what)
{
  static int closing = 0;
  unsigned char *sense = aspi_sense;
  if (s == 1)
    return;
  printf("%s returns %d, sense = %02x %02x %02x%02x %02x\n",
    what, s, sense[2], sense[12],
	 sense[16], sense[17], sense[15]);
  if (ignore)
    return;
  if (closing)
    aspi_close();
  else
  {
    closing = 1;
    cd_close(0);
  }
  exit(1);
}

int
cd_open(void)
{
  int t;
  unsigned char buf[200];
  if (cd_is_open == CDO_open)
    return;
  aspi_init();
  for (t=0; t<8*8*8; t++)
  {
    int dt;
    dt = aspi_device_type(t);
    if (dt != -1)
    {
      if (dt == DTYPE_WORM || dt == DTYPE_CROM)
      {
	memset(cdb, 0,6);
	cdb[0] = 0x12;
	cdb[4] = 250;
	aspi_exec(t, buf, 200, ASPI_RW_READ, cdb, 6);
        target = t;
        printf("scsi(%d,%d,%d) type %s : %8.8s  %8.8s\n",
	       ASPI_ID2HOSTAD(target),
	       ASPI_ID2TARGET(target),
	       ASPI_ID2LUN(target),
	       dt == DTYPE_CROM ? "CD-ROM" : "CD-WORM",
	       buf+8, buf+16);
	break;
      }
    }
  }
  if (target == 8*8*8)
  {
    fprintf(stderr, "Error: no CD-WORM devices detected\n");
    aspi_close();
    exit(0);
  }

  if (cd_is_open == CDO_init)
    atexit(cd_close_auto);
  cd_is_open = CDO_open;

  return target;
}

void
cd_close(int eject)
{
  int s;

  if (cd_is_open == CDO_close)
    return;
  cd_is_open = CDO_close;

  memset(cdb, 0, 6);
  cdb[0] = 0x1e;
  cdb[4] = 0;
  s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 6);
  check_sense(0, s, "allow media removal");
/*  printf("media tray is unlocked\n"); */

  if (eject)
  {
    memset(cdb, 0, 10);
    cdb[0] = 0xe7;
    s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 10);
    check_sense(1, s, "medium unload");
  }

  aspi_close();
}

char *
cd_addr(unsigned long a)
{
  static char buf[10];
  sprintf(buf, "%06lx:%03lx", a/blocksize, a%blocksize);
  return buf;
}
