#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include "aspi.h"
#include "scsidefs.h"
#include "cdrom.h"
     
static unsigned char cdb[12];
static int target;

unsigned char wavhdr[] = {
                      /* size of cdda+36 */
  'R', 'I', 'F', 'F', 0x00, 0x00, 0x00, 0x00,
  'W', 'A', 'V', 'E', 'f', 'm', 't', ' ',
  0x10, 0x00, 0x00, 0x00,
  0x01, 0x00, 0x02, 0x00,
  0x44, 0xac, 0x00, 0x00,
  0x10, 0xb1, 0x02, 0x00,
  0x04, 0x00, 0x10, 0x00,
                      /* size of data     */
  'd', 'a', 't', 'a', 0x00, 0x00, 0x00, 0x00
};

main(int argc, char **argv)
{
  int i, max, n;
  int h, m, s, oh;
  unsigned char buf[16384];
  int first_track, last_track;
  int track;
  int da_first=0, da_last=0;
  char *fn;
  FILE *f;
  int single_speed = 0;
  int last_was_error = 0;

  if (argc > 1 && strcmp(argv[1], "-1") == 0)
  {
    single_speed = 1;
    argc--;
    argv++;
  }

  if (argc != 3)
  {
    fprintf(stderr, "Usage: getda [track] [file]\n");
    exit(1);
  }

  track = atoi(argv[1]);
  fn = argv[2];

  target = cd_open();

  do {
    memset(cdb, 0, 6);
    s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 6);
  } while (s != 1 && (aspi_sense[12] == 0x28 || aspi_sense[12] == 0x04));
  check_sense(0, s, "test unit ready");

#if 0
  memset(cdb, 0, 6);
  cdb[0] = 0x15;
  cdb[1] = 0x10;
  cdb[4] = 4+8;
  memset(buf, 0, 4+8);
  buf[4+0] = 0x01;
  buf[4+1] = 0x06;
  buf[4+2] = 0x20; /* TB */
  buf[4+3] = 0x03; /* retries */
  s = aspi_exec(target, buf, 4+8, ASPI_RW_WRITE, cdb, 6);
  check_sense(0, s, "set recovery");
#endif

  max = 4+8*100;

  memset(cdb, 0, 10);
  cdb[0] = 0x43;
  cdb[7] = max >> 8;
  cdb[8] = max & 0xff;
  s = aspi_exec(target, buf, max, ASPI_RW_READ, cdb, 10);
  check_sense(0, s, "read TOC");

  first_track = buf[2];
  last_track = buf[3];
  max = buf[0] * 256 + buf[1];

  n = 0;
  for (i=4; i<max; i+= 8)
  {
    int start = swapl(*(unsigned long *)(buf+i+4));
    int nstart = swapl(*(unsigned long *)(buf+i+4+8));
    int len = nstart - start;
    if (buf[i+2] != 0xaa)
    {
      printf("  %2d.(%d:%02d)", buf[i+2], len/75/60, (len/75)%60);
      n++;
      if (n == 5)
      {
	n = 0;
	printf("\n");
      }
    }
    else
    {
      if (n)
	printf("\n");
    }
    if (buf[i+2] == track)
    {
      da_first = start;
      if (track > 1)
	da_first -= 75;
      da_last = nstart-1;
    }
  }

  if (da_last == 0)
  {
    printf("disk does not include that track.\n");
    exit(1);
  }

  f = fopen(fn, "wb");
  if (f == 0)
  {
    fprintf(stderr, "Cannot open file %s for writing\n", fn);
    perror("The error was");
    exit(1);
  }

  *(unsigned long *)(wavhdr+4) = (da_last - da_first + 1) * 2352 + 36;
  *(unsigned long *)(wavhdr+40) = (da_last - da_first + 1) * 2352;
  fwrite(wavhdr, 1, sizeof(wavhdr), f);

  if (single_speed)
  {
    memset(cdb, 0, 6);
    cdb[0] = 0x15;
    cdb[1] = 0x10;
    cdb[4] = 4 + 4;
    memset(buf, 0, 16);
    buf[4] = 0x31;
    buf[5] = 2;
    buf[6] = 0;
    buf[7] = 0x01; /* single/nowrite */
    s = aspi_exec(target, buf, 8, ASPI_RW_WRITE, cdb, 6);
    check_sense(0, s, "mode select");
    printf("Single speed transfer\n");
  }

  h = (da_last-da_first+1) / 75;
  printf("\nReading track %d  %d:%02d\n", track, h/60, h%60);

  h = 0;
  oh = 0;
  for (i=da_first; i<=da_last; i++)
  {
    h++;
    memset(cdb, 0, 12);
    cdb[0] = 0xd8;
    *(unsigned long *)(cdb+2) = swapl(i);
    *(unsigned long *)(cdb+6) = swapl(1);
    cdb[10] = 0;
    s = aspi_exec(target, buf, 2352, ASPI_RW_READ, cdb, 12);
    if (s != 1)
    {
      if (!last_was_error)
      {
	printf("\nRead error: %s", lb2hms(h));
	last_was_error = 1;
      }
      else
	last_was_error = 2;
      memset(buf, 0, 2352);
    }
    else if (last_was_error)
    {
      if (last_was_error == 2)
	printf(" to %s", lb2hms(h-1));
      printf("\n");
      last_was_error = 0;
    }
    fwrite(buf, 1, 2352, f);
    if (oh++ == 74)
    {
      printf("\r%s ", lb2hms(h));
      fflush(stdout);
      oh = 0;
      if (kbhit())
      {
	getkey();
	break;
      }
    }
  }
  fclose(f);

  cd_close(0);

  return 0;
}
