#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <io.h>
#include <fcntl.h>
#include <dos.h>
#include <sys/stat.h>

#include "aspi.h"
#include "scsidefs.h"
#include "cdrom.h"

static unsigned char cdb[10];
static int target;

main(int argc, char **argv)
{
  int i;
  int h, m, s;
  int dummy_mode = 1;
  unsigned char buf[16384], *sense;
  struct stat statb;
  unsigned long left_to_read, written_so_far=0;
  int input_file;
  time_t start_time, end_time, now_time, bps_start_time=0;
  int last_mb=0, this_mb;
  int bps_start_count;

  if (argc > 1 && strcmp(argv[1], "-no,really!") == 0)
  {
    dummy_mode = 0;
    argc--;
    argv++;
  }

  if (argc < 2)
  {
    printf("Usage: makecd [-no,really!] <file>\n");
    printf("       makecd -u    (unlock)\n");
    return 1;
  }
  target = cd_open();
  if (strcmp(argv[1], "-u") == 0)
  {
    memset(cdb, 0, 6);
    cdb[0] = 0x1e;
    cdb[4] = 0;
    s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 6);
    printf("media tray is unlocked\n");

    memset(cdb, 0, 10);
    cdb[0] = 0xe7;
    s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 10);
    check_sense(1, s, "medium unload");

    exit(0);
  }
  if (stat(argv[1], &statb))
  {
    printf("Cannot stat file %s\n", argv[1]);
    return 1;
  }
  left_to_read = statb.st_size;
  input_file = _open(argv[1], O_RDONLY);
  if (input_file < 0)
  {
    printf("Cannot open input file\n");
    return 1;
  }
  printf("%s bytes\n", comma(left_to_read));

  do {
    memset(cdb, 0, 6);
    s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 6);
    sense = aspi_sense;
  } while (s != 1 && sense[12] == 0x28);
  check_sense(0, s, "test unit ready");

  memset(cdb, 0, 6);
  cdb[0] = 0x1e;
  cdb[4] = 1;
  s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 6);
  check_sense(0, s, "prevent media removal");
  printf("media tray is locked\n");

  memset(cdb, 0, 6);
  cdb[0] = 0x15;
  cdb[1] = 0x10;
  cdb[4] = 4 + 4;
  memset(buf, 0, 16);
  buf[4] = 0x31;
  buf[5] = 2;
  buf[6] = 0;
  buf[7] = dummy_mode | 0x20; /* quad speed */
  s = aspi_exec(target, buf, 8, ASPI_RW_WRITE, cdb, 6);
  check_sense(0, s, "mode select");
  printf("mode select OK\n");

  memset(cdb, 0, 6);
  cdb[0] = 0x1a;
  cdb[2] = 0x31;
  cdb[4] = 4 + 8 + 4;
  memset(buf, 0, 16);
  s = aspi_exec(target, buf, 16, ASPI_RW_READ, cdb, 6);
  check_sense(0, s, "mode sense");
  printf("mode sense OK - drive config is %02x, density %02x, block size %02x%02x%02x\n",
	 buf[4+8+3],
	 buf[4+0], buf[4+5], buf[4+6], buf[4+7]);
  if (buf[4+8+3] != (0x20 | dummy_mode)
      || buf[4+5] != 0 || buf[4+6] != 0x08 || buf[4+7] != 0)
  {
    cd_close(0);
    return 1;
  }

  memset(cdb, 0, 10);
  cdb[0] = 0xe2;
  cdb[3] = 1;
  cdb[8] = 6;
  s = aspi_exec(target, buf, 6, ASPI_RW_READ, cdb, 10);
  sense = aspi_sense;
  if (s != 1 && sense[12] == 0xa3)
  {
    printf("Error: Must insert WO disk into unit.\n");
    printf("s = %d (1) and sense[2,12] = %02x,%02x (a3)\n", s, sense[2], sense[12]);
    cd_close(1);
    return 1;
  }
  check_sense(0, s, "next writable track");
  printf("Next track: %ld\n", swapl(*(unsigned long *)(buf+1)));

  if (dummy_mode == 0)
  {
    printf("Warning!  This will actually write to the disk!  OK? ");
    fflush(stdout);
    gets(buf);
    if (buf[0] != 'y')
    {
      cd_close(0);
      return 1;
    }
  }
  else
    printf("Test mode.\n");

  s = statb.st_size / 600000L;
  m = s/60;
  h = m/60;
  s %= 60;
  m %= 60;
  printf("writing data...\n\n");
  printf("\r%14s b, %2d:%02d:%02d left", comma(last_mb), h, m, s);
  fflush(stdout);

  time(&start_time);

  memset(cdb, 0, 10);
  cdb[0] = 0xe6;
  cdb[6] = 1;
  s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 10);
  check_sense(0, s, "write track");

  while (1)
  {
    long read_this_time = _read(input_file, buf, 16384);
    if (read_this_time <= 0)
      break;
    read_this_time = (read_this_time+2047) & ~2047;
    fflush(stdout);
    memset(cdb, 0, 6);
    cdb[0] = 0x0a;
    cdb[4] = read_this_time/2048;
    s = aspi_exec(target, buf, read_this_time, ASPI_RW_WRITE, cdb, 6);
    if (s != 1)
      putchar('\n');
    check_sense(0, s, "write data");
    written_so_far += read_this_time;
    this_mb = written_so_far >> 20;
    if (this_mb != last_mb)
    {
      time(&now_time);
      if (bps_start_time == 0)
      {
	time(&bps_start_time);
	bps_start_count = written_so_far;

	end_time = start_time + (now_time-start_time)
	  * (statb.st_size) / (written_so_far);
      }
      else
      {
	end_time = start_time + (now_time-bps_start_time)
	  * (statb.st_size-bps_start_count) / (written_so_far-bps_start_count);
      }
      s = end_time - now_time;
      if (s < 0)
	s = 0;
      m = s/60;
      h = m/60;
      s %= 60;
      m %= 60;
      printf("\r%14s Mb, %2d:%02d:%02d left", comma(written_so_far), h, m, s);
      fflush(stdout);
      last_mb = this_mb;
    }
  }
  printf("\n\n");

  memset(cdb, 0, 10);
  cdb[0] = 0x35;
  s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 10);
  check_sense(1, s, "flush");

  time(&end_time);

  printf("%s bytes in %ld seconds,", comma(written_so_far), end_time - start_time);
  printf(" %s bytes/sec\n", comma((written_so_far-bps_start_count) / (end_time - bps_start_time)));

  printf("closing disk...\n");
  memset(cdb, 0, 10);
  cdb[0] = 0xe9;
  cdb[8] = 0x01; /* CD-ROM TOC, single session */
  s = aspi_exec(target, 0, 0, ASPI_RW_NODATA, cdb, 10);
  check_sense(1, s, "fixation");

  cd_close(1);

  return 0;
}
